@@ -128,7 +128,7 @@ to_chars_subst (int negative, int gnu_format, uintmax_t value, size_t valsize,
   char const *minval_string;
   char const *maxval_string = STRINGIFY_BIGINT (maxval, maxbuf);
   char const *value_string;
-    
+
   if (gnu_format)
     {
       uintmax_t m = maxval + 1 ? maxval + 1 : maxval / 2 + 1;
@@ -138,7 +138,7 @@ to_chars_subst (int negative, int gnu_format, uintmax_t value, size_t valsize,
     }
   else
     minval_string = "0";
-  
+
   if (negative)
     {
       char *p = STRINGIFY_BIGINT (- value, valbuf + 1);
@@ -147,13 +147,18 @@ to_chars_subst (int negative, int gnu_format, uintmax_t value, size_t valsize,
     }
   else
     value_string = STRINGIFY_BIGINT (value, valbuf);
-	  
+
   if (substitute)
     {
       int negsub;
       uintmax_t sub = substitute (&negsub) & maxval;
-      /* FIXME: This is the only place where GNU_FORMAT differs from
-	 OLDGNU_FORMAT. Apart from this they are completely identical. */
+      /* NOTE: This is one of the few places where GNU_FORMAT differs from
+	 OLDGNU_FORMAT.  The actual differences are:
+
+	 1. In OLDGNU_FORMAT all strings in a tar header end in \0
+	 2. Incremental archives use oldgnu_header.
+	 
+	 Apart from this they are completely identical. */
       uintmax_t s = (negsub &= archive_format == GNU_FORMAT) ? - sub : sub;
       char subbuf[UINTMAX_STRSIZE_BOUND + 1];
       char *sub_string = STRINGIFY_BIGINT (s, subbuf + 1);
@@ -236,7 +241,7 @@ to_chars (int negative, uintmax_t value, size_t valsize,
     }
   else
     substitute = NULL; /* No substitution for formats, other than GNU */
-  
+
   return to_chars_subst (negative, gnu_format, value, valsize, substitute,
 			 where, size, type);
 }
@@ -291,7 +296,8 @@ mode_to_chars (mode_t v, char *p, size_t s)
       && S_IROTH == TOREAD && S_IWOTH == TOWRITE && S_IXOTH == TOEXEC
       && archive_format != POSIX_FORMAT
       && archive_format != USTAR_FORMAT
-      && archive_format != GNU_FORMAT)
+      && archive_format != GNU_FORMAT
+      && archive_format != OLDGNU_FORMAT)
     {
       negative = v < 0;
       u = v;
@@ -380,7 +386,7 @@ bool
 file_dumpable_p (struct tar_stat_info *st)
 {
   if (dev_null_output)
-    return totals_option && sparse_option && sparse_file_p (st);
+    return totals_option && sparse_option && ST_IS_SPARSE (st->stat);
   return !(st->archive_file_size == 0
 	   && (st->stat.st_mode & MODE_R) == MODE_R);
 }
@@ -663,7 +669,8 @@ start_header (struct tar_stat_info *st)
   if (mode_option)
     st->stat.st_mode =
       ((st->stat.st_mode & ~MODE_ALL)
-       | mode_adjust (st->stat.st_mode, mode_option, initial_umask));
+       | mode_adjust (st->stat.st_mode, S_ISDIR (st->stat.st_mode) != 0,
+		      initial_umask, mode_option, NULL));
 
   /* Paul Eggert tried the trivial test ($WRITER cf a b; $READER tvf a)
      for a few tars and came up with the following interoperability
@@ -730,7 +737,7 @@ start_header (struct tar_stat_info *st)
   }
 
   {
-    struct timespec mtime = st->mtime;
+    struct timespec mtime = set_mtime_option ? mtime_option : st->mtime;
     if (archive_format == POSIX_FORMAT)
       {
 	if (MAX_OCTAL_VAL (header->header.mtime) < mtime.tv_sec
@@ -967,8 +974,8 @@ dump_regular_file (int fd, struct tar_stat_info *st)
 			   size_left),
 		 quotearg_colon (st->orig_file_name),
 		 STRINGIFY_BIGINT (size_left, buf)));
-	  if (! ignore_failed_read_option)
-	    exit_status = TAREXIT_FAILURE;
+	  if (! ignore_failed_read_option) 
+	    exit_status = TAREXIT_DIFFERS;
 	  pad_archive (size_left - (bufsize-count));
 	  return dump_status_short;
 	}
@@ -1392,6 +1399,7 @@ dump_file0 (struct tar_stat_info *st, const char *p,
 {
   union block *header;
   char type;
+  off_t original_size;
   struct timespec original_ctime;
   struct timespec restore_times[2];
   off_t block_ordinal = -1;
@@ -1411,7 +1419,7 @@ dump_file0 (struct tar_stat_info *st, const char *p,
       stat_diag (p);
       return;
     }
-  st->archive_file_size = st->stat.st_size;
+  st->archive_file_size = original_size = st->stat.st_size;
   st->atime = restore_times[0] = get_stat_atime (&st->stat);
   st->mtime = restore_times[1] = get_stat_mtime (&st->stat);
   st->ctime = original_ctime = get_stat_ctime (&st->stat);
@@ -1499,7 +1507,7 @@ dump_file0 (struct tar_stat_info *st, const char *p,
 	{
 	  enum dump_status status;
 
-	  if (fd != -1 && sparse_option && sparse_file_p (st))
+	  if (fd != -1 && sparse_option && ST_IS_SPARSE (st->stat))
 	    {
 	      status = sparse_dump_file (fd, st);
 	      if (status == dump_status_not_implemented)
@@ -1523,7 +1531,7 @@ dump_file0 (struct tar_stat_info *st, const char *p,
 	    }
 
 	  file_count_links (st);
-	  
+
 	  ok = status == dump_status_ok;
 	}
 
@@ -1548,9 +1556,14 @@ dump_file0 (struct tar_stat_info *st, const char *p,
 
       if (ok)
 	{
-	  if (timespec_cmp (get_stat_ctime (&final_stat), original_ctime) != 0)
-	    WARN ((0, 0, _("%s: file changed as we read it"),
-		   quotearg_colon (p)));
+	  if (timespec_cmp (get_stat_ctime (&final_stat), original_ctime) != 0
+	      || original_size < final_stat.st_size)
+	    {
+	      WARN ((0, 0, _("%s: file changed as we read it"),
+		     quotearg_colon (p)));
+	      if (exit_status == TAREXIT_SUCCESS)
+		exit_status = TAREXIT_DIFFERS;
+	    }
 	  else if (atime_preserve_option == replace_atime_preserve
 		   && set_file_atime (fd, p, restore_times) != 0)
 	    utime_error (p);
